@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2010 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2015 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #if USE_ONIGURUMA
-#  include <onigposix.h>
+#  include <oniguruma.h>
 #elif HAVE_REGEX_H
 #  include <regex.h>
 #endif
@@ -133,6 +133,8 @@ gint filter_apply_msginfo(GSList *fltlist, MsgInfo *msginfo,
 			g_warning("filter_match_rule() returned error (code: %d)\n", fltinfo->error);
 		}
 		if (matched) {
+			debug_print("filter-log: %s: rule [%s] matched\n",
+				    G_STRFUNC, rule->name ? rule->name : "(No name)");
 			ret = filter_action_exec(rule, msginfo, file, fltinfo);
 			if (ret < 0) {
 				g_warning("filter_action_exec() returned error (code: %d)\n", fltinfo->error);
@@ -160,6 +162,7 @@ gint filter_action_exec(FilterRule *rule, MsgInfo *msginfo, const gchar *file,
 	GSList *cur;
 	gchar *cmdline;
 	gboolean copy_to_self = FALSE;
+	gint ret;
 
 	g_return_val_if_fail(rule != NULL, -1);
 	g_return_val_if_fail(msginfo != NULL, -1);
@@ -198,14 +201,28 @@ gint filter_action_exec(FilterRule *rule, MsgInfo *msginfo, const gchar *file,
 		case FLT_ACTION_EXEC:
 			cmdline = g_strconcat(action->str_value, " \"", file,
 					      "\"", NULL);
-			execute_command_line(cmdline, FALSE);
+			ret = execute_command_line(cmdline, FALSE);
+			fltinfo->last_exec_exit_status = ret;
+			if (ret == -1) {
+				fltinfo->error = FLT_ERROR_EXEC_FAILED;
+				g_warning("filter_action_exec: cannot execute command: %s", cmdline);
+				g_free(cmdline);
+				return -1;
+			}
 			g_free(cmdline);
 			fltinfo->actions[action->type] = TRUE;
 			break;
 		case FLT_ACTION_EXEC_ASYNC:
 			cmdline = g_strconcat(action->str_value, " \"", file,
 					      "\"", NULL);
-			execute_command_line(cmdline, TRUE);
+			ret = execute_command_line(cmdline, TRUE);
+			fltinfo->last_exec_exit_status = ret;
+			if (ret == -1) {
+				fltinfo->error = FLT_ERROR_EXEC_FAILED;
+				g_warning("filter_action_exec: cannot execute command: %s", cmdline);
+				g_free(cmdline);
+				return -1;
+			}
 			g_free(cmdline);
 			fltinfo->actions[action->type] = TRUE;
 			break;
@@ -318,13 +335,31 @@ gint filter_action_exec(FilterRule *rule, MsgInfo *msginfo, const gchar *file,
 
 static gboolean strmatch_regex(const gchar *haystack, const gchar *needle)
 {
-#if defined(USE_ONIGURUMA) || defined(HAVE_REGCOMP)
+#ifdef USE_ONIGURUMA
+	gint ret = 0;
+	OnigRegex reg;
+	OnigErrorInfo err_info;
+	const UChar *ptn = (const UChar *)needle;
+	const UChar *str = (const UChar *)haystack;
+
+	ret = onig_new(&reg, ptn, ptn + strlen(needle),
+		       ONIG_OPTION_IGNORECASE|ONIG_OPTION_EXTEND,
+		       ONIG_ENCODING_UTF8, ONIG_SYNTAX_POSIX_EXTENDED,
+		       &err_info);
+	if (ret != ONIG_NORMAL)
+		return FALSE;
+
+	ret = onig_match(reg, str, str + strlen(haystack), str, NULL, 0);
+	onig_free(reg);
+
+	if (ret >= 0)
+		return TRUE;
+	else
+		return FALSE;
+#elif defined(HAVE_REGCOMP)
 	gint ret = 0;
 	regex_t preg;
 
-#if USE_ONIGURUMA
-	reg_set_encoding(REG_POSIX_ENCODING_UTF8);
-#endif
 	ret = regcomp(&preg, needle, REG_ICASE|REG_EXTENDED);
 	if (ret != 0)
 		return FALSE;
@@ -335,8 +370,10 @@ static gboolean strmatch_regex(const gchar *haystack, const gchar *needle)
 	if (ret == 0)
 		return TRUE;
 	else
-#endif
 		return FALSE;
+#else
+	return FALSE;
+#endif
 }
 
 gboolean filter_match_rule(FilterRule *rule, MsgInfo *msginfo, GSList *hlist,
@@ -366,20 +403,66 @@ gboolean filter_match_rule(FilterRule *rule, MsgInfo *msginfo, GSList *hlist,
 	if (rule->bool_op == FLT_AND) {
 		for (cur = rule->cond_list; cur != NULL; cur = cur->next) {
 			cond = (FilterCond *)cur->data;
-			matched = filter_match_cond(cond, msginfo, hlist,
-						    fltinfo);
-			if (matched == FALSE)
-				return FALSE;
+			if (cond->type >= FLT_COND_SIZE_GREATER) {
+				matched = filter_match_cond
+					(cond, msginfo, hlist, fltinfo);
+				if (matched == FALSE)
+					return FALSE;
+			}
+		}
+
+		for (cur = rule->cond_list; cur != NULL; cur = cur->next) {
+			cond = (FilterCond *)cur->data;
+			if (cond->type <= FLT_COND_TO_OR_CC) {
+				matched = filter_match_cond
+					(cond, msginfo, hlist, fltinfo);
+				if (matched == FALSE)
+					return FALSE;
+			}
+		}
+
+		for (cur = rule->cond_list; cur != NULL; cur = cur->next) {
+			cond = (FilterCond *)cur->data;
+			if (cond->type == FLT_COND_BODY ||
+			    cond->type == FLT_COND_CMD_TEST) {
+				matched = filter_match_cond
+					(cond, msginfo, hlist, fltinfo);
+				if (matched == FALSE)
+					return FALSE;
+			}
 		}
 
 		return TRUE;
 	} else if (rule->bool_op == FLT_OR) {
 		for (cur = rule->cond_list; cur != NULL; cur = cur->next) {
 			cond = (FilterCond *)cur->data;
-			matched = filter_match_cond(cond, msginfo, hlist,
-						    fltinfo);
-			if (matched == TRUE)
-				return TRUE;
+			if (cond->type >= FLT_COND_SIZE_GREATER) {
+				matched = filter_match_cond
+					(cond, msginfo, hlist, fltinfo);
+				if (matched == TRUE)
+					return TRUE;
+			}
+		}
+
+		for (cur = rule->cond_list; cur != NULL; cur = cur->next) {
+			cond = (FilterCond *)cur->data;
+			if (cond->type <= FLT_COND_TO_OR_CC) {
+				matched = filter_match_cond
+					(cond, msginfo, hlist, fltinfo);
+				if (matched == TRUE)
+					return TRUE;
+			}
+		}
+
+		for (cur = rule->cond_list; cur != NULL; cur = cur->next) {
+			cond = (FilterCond *)cur->data;
+			if (cond->type == FLT_COND_BODY ||
+			    cond->type == FLT_COND_CMD_TEST) {
+				matched = filter_match_cond
+					(cond, msginfo, hlist, fltinfo);
+				if (matched == TRUE)
+					return TRUE;
+			}
 		}
 
 		return FALSE;
@@ -393,6 +476,8 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 {
 	gint ret;
 	gboolean matched = FALSE;
+	gboolean not_match = FALSE;
+	gint64 timediff = 0;
 	gchar *file;
 	gchar *cmdline;
 	PrefsAccount *cond_ac;
@@ -420,6 +505,7 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 			return FALSE;
 		cmdline = g_strconcat(cond->str_value, " \"", file, "\"", NULL);
 		ret = execute_command_line_async_wait(cmdline);
+		fltinfo->last_exec_exit_status = ret;
 		matched = (ret == 0);
 		if (ret == -1)
 			fltinfo->error = FLT_ERROR_EXEC_FAILED;
@@ -430,8 +516,8 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 		matched = (msginfo->size > cond->int_value * 1024);
 		break;
 	case FLT_COND_AGE_GREATER:
-		matched = (time(NULL) - msginfo->date_t >
-				cond->int_value * 24 * 60 * 60);
+		timediff = time(NULL) - msginfo->date_t;
+		matched = (timediff > cond->int_value * 24 * 60 * 60);
 		break;
 	case FLT_COND_UNREAD:
 		matched = MSG_IS_UNREAD(msginfo->flags);
@@ -456,8 +542,47 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 		return FALSE;
 	}
 
-	if (FLT_IS_NOT_MATCH(cond->match_flag))
+	if (FLT_IS_NOT_MATCH(cond->match_flag)) {
+		not_match = TRUE;
 		matched = !matched;
+	}
+
+	if (matched && get_debug_mode()) {
+		gchar *sv = cond->str_value ? cond->str_value : "";
+		gchar *nm = not_match ? " (reverse match)" : "";
+
+		switch (cond->type) {
+		case FLT_COND_BODY:
+			debug_print("filter-log: %s: BODY, str_value: [%s]%s\n", G_STRFUNC, sv, nm);
+			break;
+		case FLT_COND_CMD_TEST:
+			debug_print("filter-log: %s: CMD_TEST, str_value: [%s]%s\n", G_STRFUNC, sv, nm);
+			break;
+		case FLT_COND_SIZE_GREATER:
+			debug_print("filter-log: %s: SIZE_GREATER: %u %s %d (KB)%s\n", G_STRFUNC, msginfo->size, not_match ? "<=" : ">", cond->int_value, nm);
+			break;
+		case FLT_COND_AGE_GREATER:
+			debug_print("filter-log: %s: AGE_GREATER: %lld (sec) %s %d (day)%s\n", G_STRFUNC, timediff, not_match ? "<=" : ">", cond->int_value, nm);
+			break;
+		case FLT_COND_UNREAD:
+			debug_print("filter-log: %s: UNREAD%s\n", G_STRFUNC, nm);
+			break;
+		case FLT_COND_MARK:
+			debug_print("filter-log: %s: MARK%s\n", G_STRFUNC, nm);
+			break;
+		case FLT_COND_COLOR_LABEL:
+			debug_print("filter-log: %s: COLOR_LABEL%s\n", G_STRFUNC, nm);
+			break;
+		case FLT_COND_MIME:
+			debug_print("filter-log: %s: MIME%s\n", G_STRFUNC, nm);
+			break;
+		case FLT_COND_ACCOUNT:
+			debug_print("filter-log: %s: ACCOUNT [%d]%s\n", G_STRFUNC, cond->int_value, nm);
+			break;
+		default:
+			break;
+		}
+	}
 
 	return matched;
 }
@@ -465,6 +590,7 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 static gboolean filter_match_header_cond(FilterCond *cond, GSList *hlist)
 {
 	gboolean matched = FALSE;
+	gboolean not_match = FALSE;
 	GSList *cur;
 	Header *header;
 
@@ -503,8 +629,29 @@ static gboolean filter_match_header_cond(FilterCond *cond, GSList *hlist)
 			break;
 	}
 
-	if (FLT_IS_NOT_MATCH(cond->match_flag))
+	if (FLT_IS_NOT_MATCH(cond->match_flag)) {
+		not_match = TRUE;
 		matched = !matched;
+	}
+
+	if (matched && get_debug_mode()) {
+		gchar *sv = cond->str_value ? cond->str_value : "";
+		gchar *nm = not_match ? " (reverse match)" : "";
+
+		switch (cond->type) {
+		case FLT_COND_HEADER:
+			debug_print("filter-log: %s: HEADER [%s], str_value: [%s]%s\n", G_STRFUNC, cond->header_name, sv, nm);
+			break;
+		case FLT_COND_ANY_HEADER:
+			debug_print("filter-log: %s: ANY_HEADER, str_value: [%s]%s\n", G_STRFUNC, sv, nm);
+			break;
+		case FLT_COND_TO_OR_CC:
+			debug_print("filter-log: %s: TO_OR_CC, str_value: [%s]%s\n", G_STRFUNC, sv, nm);
+			break;
+		default:
+			break;
+		}
+	}
 
 	return matched;
 }
@@ -513,6 +660,7 @@ static gboolean filter_match_in_addressbook(FilterCond *cond, GSList *hlist,
 					    FilterInfo *fltinfo)
 {
 	gboolean matched = FALSE;
+	gboolean not_match = FALSE;
 	GSList *cur;
 	Header *header;
 
@@ -542,8 +690,25 @@ static gboolean filter_match_in_addressbook(FilterCond *cond, GSList *hlist,
 			break;
 	}
 
-	if (FLT_IS_NOT_MATCH(cond->match_flag))
+	if (FLT_IS_NOT_MATCH(cond->match_flag)) {
+		not_match = TRUE;
 		matched = !matched;
+	}
+
+	if (matched && get_debug_mode()) {
+		gchar *nm = not_match ? " (reverse match)" : "";
+
+		switch (cond->type) {
+		case FLT_COND_HEADER:
+			debug_print("filter-log: %s: HEADER [%s], IN_ADDRESSBOOK%s\n", G_STRFUNC, cond->header_name, nm);
+			break;
+		case FLT_COND_TO_OR_CC:
+			debug_print("filter-log: %s: TO_OR_CC, IN_ADDRESSBOOK%s\n", G_STRFUNC, nm);
+			break;
+		default:
+			break;
+		}
+	}
 
 	return matched;
 }
@@ -1317,7 +1482,8 @@ FilterCond *filter_cond_new(FilterCondType type,
 		cond->header_name = NULL;
 
 	cond->str_value = (value && *value) ? g_strdup(value) : NULL;
-	if (type == FLT_COND_SIZE_GREATER || type == FLT_COND_AGE_GREATER)
+	if (type == FLT_COND_SIZE_GREATER || type == FLT_COND_AGE_GREATER ||
+	    type == FLT_COND_ACCOUNT)
 		cond->int_value = atoi(value);
 	else
 		cond->int_value = 0;
@@ -1366,10 +1532,118 @@ FilterInfo *filter_info_new(void)
 	fltinfo->move_dest = NULL;
 	fltinfo->drop_done = FALSE;
 	fltinfo->error = FLT_ERROR_OK;
+	fltinfo->last_exec_exit_status = 0;
 
 	return fltinfo;
 }
 
+FilterRule *filter_junk_rule_create(PrefsAccount *account,
+				    FolderItem *default_junk,
+				    gboolean is_manual)
+{
+	FilterRule *rule;
+	FilterCond *cond;
+	FilterAction *action;
+	GSList *cond_list = NULL, *action_list = NULL;
+	gchar *junk_id = NULL;
+	FolderItem *item = NULL;
+
+	if (!prefs_common.junk_classify_cmd)
+		return NULL;
+
+	if (prefs_common.junk_folder)
+		item = folder_find_item_from_identifier(prefs_common.junk_folder);
+
+	if (!item && account) {
+		Folder *folder = NULL;
+		GList *list;
+
+		/* find most suitable Junk folder for account */
+
+		if (account->inbox && *account->inbox == '#') {
+			FolderItem *inbox;
+			inbox = folder_find_item_from_identifier(account->inbox);
+			if (inbox) {
+				folder = inbox->folder;
+				if (folder)
+					item = folder_get_junk(folder);
+			}
+		}
+		if (!item) {
+			folder = FOLDER(account->folder);
+			if (folder)
+				item = folder_get_junk(folder);
+		}
+		if (!item) {
+			for (list = folder_get_list(); list != NULL; list = list->next) {
+				folder = FOLDER(list->data);
+				if (FOLDER_IS_LOCAL(folder)) {
+					if (folder->account == account)
+						item = folder_get_junk(folder);
+					if (!item && folder->node) {
+						item = FOLDER_ITEM(folder->node->data);
+						if (item && item->account == account && item->folder)
+							item = folder_get_junk(item->folder);
+						else
+							item = NULL;
+					}
+				}
+				if (item)
+					break;
+			}
+		}
+	}
+
+	if (!item)
+		item = default_junk;
+	if (!item)
+		item = folder_get_default_junk();
+	if (!item)
+		return NULL;
+	junk_id = folder_item_get_identifier(item);
+	if (!junk_id)
+		return NULL;
+
+	debug_print("filter_junk_rule_create: junk folder: %s\n",
+		    junk_id);
+
+	if (prefs_common.nofilter_junk_sender_in_book) {
+		cond = filter_cond_new(FLT_COND_HEADER, FLT_IN_ADDRESSBOOK,
+				       FLT_NOT_MATCH, "From", NULL);
+		cond_list = g_slist_append(cond_list, cond);
+	}
+
+	cond = filter_cond_new(FLT_COND_CMD_TEST, 0, 0, NULL,
+			       prefs_common.junk_classify_cmd);
+	cond_list = g_slist_append(cond_list, cond);
+
+	if (prefs_common.delete_junk_on_recv && !is_manual) {
+		action = filter_action_new(FLT_ACTION_COPY, junk_id);
+		action_list = g_slist_append(NULL, action);
+		action = filter_action_new(FLT_ACTION_DELETE, NULL);
+		action_list = g_slist_append(action_list, action);
+	} else {
+		action = filter_action_new(FLT_ACTION_MOVE, junk_id);
+		action_list = g_slist_append(NULL, action);
+	}
+
+	if (prefs_common.mark_junk_as_read) {
+		action = filter_action_new(FLT_ACTION_MARK_READ, NULL);
+		action_list = g_slist_append(action_list, action);
+	}
+
+	if (is_manual)
+		rule = filter_rule_new(_("Junk mail filter (manual)"), FLT_AND,
+				       cond_list, action_list);
+	else
+		rule = filter_rule_new(_("Junk mail filter"), FLT_AND,
+				       cond_list, action_list);
+
+	g_free(junk_id);
+
+        return rule;
+}
+
 void filter_rule_rename_dest_path(FilterRule *rule, const gchar *old_path,
 				  const gchar *new_path)
 {
