@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2013 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2014 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -297,6 +297,8 @@ int main(int argc, char *argv[])
 
 	gtkut_widget_init();
 
+	gtkut_get_dpi();
+
 	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "icons", NULL);
 	if (is_dir_exist(path)) {
 		debug_print("icon theme dir: %s\n", path);
@@ -467,38 +469,66 @@ static void read_ini_file(void)
 #if defined(G_OS_WIN32) || defined(__APPLE__)
 static void fix_font_setting(void)
 {
-	const gchar *str = NULL;
+	gchar *str = NULL;
+	gchar **strv = NULL;
+	gchar *style = NULL;
+	gchar *size = NULL;
+	gchar *suffix = NULL;
 
 	if (!conv_is_ja_locale())
 		return;
 
 	if (prefs_common.textfont &&
 	    strcmp(prefs_common.textfont, DEFAULT_MESSAGE_FONT) != 0) {
+		guint len;
+
 		if (gtkut_font_can_load(prefs_common.textfont)) {
 			debug_print("font '%s' load ok\n", prefs_common.textfont);
 			return;
 		}
 		debug_print("font '%s' load failed\n", prefs_common.textfont);
+		debug_print("fixing prefs_common.textfont setting\n");
+
+		strv = g_strsplit(prefs_common.textfont, " ", -1);
+		len = g_strv_length(strv);
+		if (len > 0) {
+			if (g_ascii_isdigit(strv[len - 1][0]))
+				size = g_strdup(strv[len - 1]);
+			if (len > 2) {
+				if (g_ascii_strcasecmp(strv[len - 2], "Bold") == 0)
+					style = g_strdup(strv[len - 2]);
+			}
+		}
+		g_strfreev(strv);
 	}
 
-	debug_print("fixing prefs_common.textfont setting\n");
-
+	if (style && size)
+		suffix = g_strconcat(style, " ", size, NULL);
+	else if (size)
+		suffix = g_strdup(size);
+	else
+		suffix = g_strdup("12");
+	g_free(style);	
+	g_free(size);
 #ifdef G_OS_WIN32
-	str = "MS Gothic 12";
+	str = g_strconcat("MS Gothic ", suffix, NULL);
 #else /* __APPLE__ */
-	str = "Hiragino Kaku Gothic Pro Light 13";
+	str = g_strconcat("Hiragino Kaku Gothic Pro ", suffix, NULL);
 #endif
 
 	if (!gtkut_font_can_load(str)) {
 #ifdef G_OS_WIN32
 		debug_print("font '%s' load failed\n", str);
-		str = "\xef\xbc\xad\xef\xbc\xb3 \xe3\x82\xb4\xe3\x82\xb7\xe3\x83\x83\xe3\x82\xaf 12";
+		g_free(str);
+		str = g_strconcat("\xef\xbc\xad\xef\xbc\xb3 \xe3\x82\xb4\xe3\x82\xb7\xe3\x83\x83\xe3\x82\xaf ", suffix, NULL);
 		if (!gtkut_font_can_load(str)) {
 			debug_print("font '%s' load failed\n", str);
+			g_free(str);
 			str = NULL;
 		}
 #else /* __APPLE__ */
 		debug_print("font '%s' load failed\n", str);
+		g_free(str);
 		str = NULL;
 #endif
 	}
@@ -507,8 +537,11 @@ static void fix_font_setting(void)
 		debug_print("font '%s' load ok\n", str);
 		g_free(prefs_common.textfont);
 		prefs_common.textfont = g_strdup(str);
+		g_free(str);
 	} else
 		g_warning("failed to load text font!");
+
+	g_free(suffix);
 }
 #endif
 
@@ -629,6 +662,15 @@ static void parse_cmd_opt(int argc, char *argv[])
 			}
 		} else if (!strncmp(argv[i], "--safe-mode", 11)) {
 			cmd.safe_mode = TRUE;
+		} else if (!strncmp(argv[i], "--dpi", 5)) {
+			if (argv[i + 1]) {
+				gdouble dpi;
+
+				dpi = strtod(argv[i + 1], NULL);
+				if (dpi > 0.0)
+					gtkut_set_dpi(dpi);
+				i++;
+			}
 		} else if (!strncmp(argv[i], "--exit", 6)) {
 			cmd.exit = TRUE;
 		} else if (!strncmp(argv[i], "--help", 6)) {
@@ -653,6 +695,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 #ifdef G_OS_WIN32
 			g_print("%s\n", _("  --ipcport portnum      specify port for IPC remote commands"));
 #endif
+			g_print("%s\n", _("  --dpi dpinum           force DPI"));
 			g_print("%s\n", _("  --exit                 exit Sylpheed"));
 			g_print("%s\n", _("  --debug                debug mode"));
 			g_print("%s\n", _("  --safe-mode            safe mode"));
