@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2014 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@
 #include "alertpanel.h"
 #include "smtp.h"
 #include "imap.h"
+#include "plugin.h"
 
 static gboolean cancelled;
 static gboolean new_account;
@@ -124,6 +125,7 @@ static struct Compose {
 	GtkWidget *sig_text;
 	GtkTextBuffer *sig_buffer;
 	GtkWidget *sigpath_entry;
+	GtkWidget *sig_before_quote_chkbtn;
 
 	GtkWidget *autocc_chkbtn;
 	GtkWidget *autocc_entry;
@@ -140,6 +142,7 @@ static struct Privacy {
 	GtkWidget *default_sign_chkbtn;
 	GtkWidget *default_encrypt_chkbtn;
 	GtkWidget *encrypt_reply_chkbtn;
+	GtkWidget *encrypt_to_self_chkbtn;
 	GtkWidget *ascii_armored_chkbtn;
 	GtkWidget *clearsign_chkbtn;
 	GtkWidget *defaultkey_radiobtn;
@@ -174,6 +177,18 @@ static struct SSLPrefs {
 } ssl;
 #endif /* USE_SSL */
 
+static struct ProxyPrefs {
+	GtkWidget *socks_chkbtn;
+	GtkWidget *socks4_radiobtn;
+	GtkWidget *socks5_radiobtn;
+	GtkWidget *socks_host_entry;
+	GtkWidget *socks_port_entry;
+	GtkWidget *socks_auth_chkbtn;
+	GtkWidget *socks_name_entry;
+	GtkWidget *socks_pass_entry;
+	GtkWidget *socks_send_chkbtn;
+} p_proxy;
+
 static struct Advanced {
 	GtkWidget *smtpport_chkbtn;
 	GtkWidget *smtpport_entry;
@@ -306,6 +321,8 @@ static PrefsUIData ui_data[] = {
 	 prefs_set_data_from_entry, prefs_set_entry},
 	{"signature_text", &compose.sig_text,
 	 prefs_set_data_from_text, prefs_set_text},
+	{"signature_before_quote", &compose.sig_before_quote_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"set_autocc", &compose.autocc_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"auto_cc", &compose.autocc_entry,
@@ -327,6 +344,8 @@ static PrefsUIData ui_data[] = {
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"encrypt_reply", &privacy.encrypt_reply_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"encrypt_to_self", &privacy.encrypt_to_self_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"ascii_armored", &privacy.ascii_armored_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"clearsign", &privacy.clearsign_chkbtn,
@@ -356,6 +375,25 @@ static PrefsUIData ui_data[] = {
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 #endif /* USE_SSL */
 
+	/* Proxy */
+	{"use_socks", &p_proxy.socks_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"socks_type", &p_proxy.socks4_radiobtn,
+	 prefs_account_enum_set_data_from_radiobtn,
+	 prefs_account_enum_set_radiobtn},
+	{"proxy_host", &p_proxy.socks_host_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+	{"proxy_port", &p_proxy.socks_port_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+	{"use_proxy_auth", &p_proxy.socks_auth_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"proxy_name", &p_proxy.socks_name_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+	{"proxy_pass", &p_proxy.socks_pass_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+	{"use_socks_for_send", &p_proxy.socks_send_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
 	/* Advanced */
 	{"set_smtpport", &advanced.smtpport_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
@@ -377,10 +415,12 @@ static PrefsUIData ui_data[] = {
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"domain", &advanced.domain_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
+
 	{"imap_directory", &advanced.imapdir_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
 	{"imap_clear_cache_on_exit", &advanced.clear_cache_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+
 	{"set_sent_folder", &advanced.sent_folder_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"sent_folder", &advanced.sent_folder_entry,
@@ -412,6 +452,7 @@ static void prefs_account_privacy_create	(void);
 #if USE_SSL
 static void prefs_account_ssl_create		(void);
 #endif /* USE_SSL */
+static void prefs_account_proxy_create		(void);
 static void prefs_account_advanced_create	(void);
 
 static void prefs_account_select_folder_cb	(GtkWidget	*widget,
@@ -513,7 +554,10 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 	}
 
 	gtk_widget_show(dialog.window);
+	syl_plugin_signal_emit("prefs-account-open", new_account ? NULL : ac_prefs, dialog.window);
+
 	gtk_main();
+
 	gtk_widget_hide(dialog.window);
 
 	inc_unlock();
@@ -564,6 +608,8 @@ static void prefs_account_create(void)
 	prefs_account_ssl_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("SSL"), page++);
 #endif /* USE_SSL */
+	prefs_account_proxy_create();
+	SET_NOTEBOOK_LABEL(dialog.notebook, _("Proxy"), page++);
 	prefs_account_advanced_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Advanced"), page++);
 
@@ -1016,7 +1062,7 @@ static void prefs_account_receive_create(void)
 	gtk_label_set_justify (GTK_LABEL (maxarticle_label), GTK_JUSTIFY_LEFT);
 
 	maxarticle_spinbtn_adj =
-		gtk_adjustment_new (300, 0, 10000, 10, 100, 100);
+		gtk_adjustment_new (300, 0, 10000, 10, 100, 0);
 	maxarticle_spinbtn = gtk_spin_button_new
 		(GTK_ADJUSTMENT (maxarticle_spinbtn_adj), 10, 0);
 	gtk_widget_show (maxarticle_spinbtn);
@@ -1233,6 +1279,7 @@ static void prefs_account_compose_create(void)
 	GtkWidget *sigfile_radiobtn;
 	GtkWidget *sigcmd_radiobtn;
 	GtkWidget *sigpath_entry;
+	GtkWidget *sig_before_quote_chkbtn;
 	GtkWidget *frame;
 	GtkWidget *table;
 	GtkWidget *autocc_chkbtn;
@@ -1300,6 +1347,9 @@ static void prefs_account_compose_create(void)
 	gtk_widget_show (sigpath_entry);
 	gtk_box_pack_start (GTK_BOX (sig_vbox), sigpath_entry, TRUE, TRUE, 0);
 
+	PACK_CHECK_BUTTON (sig_vbox, sig_before_quote_chkbtn,
+			   _("Put signature before quote (not recommended)"));
+
 	SET_TOGGLE_SENSITIVITY (sig_radiobtn, sig_text);
 	SET_TOGGLE_SENSITIVITY (sigfile_radiobtn, sigpath_entry);
 	SET_TOGGLE_SENSITIVITY (sigcmd_radiobtn, sigpath_entry);
@@ -1364,6 +1414,8 @@ static void prefs_account_compose_create(void)
 
 	compose.sig_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(sig_text));
 
+	compose.sig_before_quote_chkbtn = sig_before_quote_chkbtn;
+
 	compose.autocc_chkbtn      = autocc_chkbtn;
 	compose.autocc_entry       = autocc_entry;
 	compose.autobcc_chkbtn     = autobcc_chkbtn;
@@ -1383,6 +1435,7 @@ static void prefs_account_privacy_create(void)
 	GtkWidget *default_sign_chkbtn;
 	GtkWidget *default_encrypt_chkbtn;
 	GtkWidget *encrypt_reply_chkbtn;
+	GtkWidget *encrypt_to_self_chkbtn;
 	GtkWidget *ascii_armored_chkbtn;
 	GtkWidget *clearsign_chkbtn;
 	GtkWidget *defaultkey_radiobtn;
@@ -1405,6 +1458,8 @@ static void prefs_account_privacy_create(void)
 			   _("PGP encrypt message by default"));
 	PACK_CHECK_BUTTON (vbox2, encrypt_reply_chkbtn,
 			   _("Encrypt when replying to encrypted message"));
+	PACK_CHECK_BUTTON (vbox2, encrypt_to_self_chkbtn,
+			   _("Add my own key to the recipients list on encryption"));
 	PACK_CHECK_BUTTON (vbox2, ascii_armored_chkbtn,
 			   _("Use ASCII-armored format for encryption"));
 	PACK_CHECK_BUTTON (vbox2, clearsign_chkbtn,
@@ -1413,7 +1468,7 @@ static void prefs_account_privacy_create(void)
 			  G_CALLBACK (prefs_account_ascii_armored_warning),
 			  NULL);
 
-	PACK_FRAME (vbox1, frame1, _("Sign key"));
+	PACK_FRAME (vbox1, frame1, _("Sign / Encryption key"));
 
 	vbox2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox2);
@@ -1469,6 +1524,7 @@ static void prefs_account_privacy_create(void)
 	privacy.default_sign_chkbtn    = default_sign_chkbtn;
 	privacy.default_encrypt_chkbtn = default_encrypt_chkbtn;
 	privacy.encrypt_reply_chkbtn   = encrypt_reply_chkbtn;
+	privacy.encrypt_to_self_chkbtn = encrypt_to_self_chkbtn;
 	privacy.ascii_armored_chkbtn   = ascii_armored_chkbtn;
 	privacy.clearsign_chkbtn       = clearsign_chkbtn;
 	privacy.defaultkey_radiobtn    = defaultkey_radiobtn;
@@ -1478,8 +1534,6 @@ static void prefs_account_privacy_create(void)
 }
 #endif /* USE_GPGME */
 
-#if USE_SSL
-
 #define CREATE_RADIO_BUTTON(box, btn, btn_p, label, data)		\
 {									\
 	btn = gtk_radio_button_new_with_label_from_widget		\
@@ -1505,6 +1559,7 @@ static void prefs_account_privacy_create(void)
 	CREATE_RADIO_BUTTON(box, btn3, btn1, btn3_label, btn3_data);	\
 }
 
+#if USE_SSL
 static void pop_ssltunnel_toggled(GtkToggleButton *button, gpointer data)
 {
 	if (gtk_toggle_button_get_active
@@ -1705,11 +1760,121 @@ static void prefs_account_ssl_create(void)
 	ssl.use_nonblocking_ssl_chkbtn = use_nonblocking_ssl_chkbtn;
 }
 
-#undef CREATE_RADIO_BUTTONS
-#undef CREATE_RADIO_BUTTON
-
 #endif /* USE_SSL */
 
+static void prefs_account_proxy_create(void)
+{
+	GtkWidget *vbox1;
+	GtkWidget *vbox2;
+	GtkWidget *socks_frame;
+	GtkWidget *socks_chkbtn;
+	GtkWidget *hbox2;
+	GtkWidget *label;
+	GtkWidget *socks4_radiobtn;
+	GtkWidget *socks5_radiobtn;
+	GtkWidget *socks_host_entry;
+	GtkWidget *socks_port_entry;
+	GtkWidget *vbox4;
+	GtkWidget *socks_auth_chkbtn;
+	GtkWidget *socks_name_entry;
+	GtkWidget *socks_pass_entry;
+	GtkWidget *socks_send_chkbtn;
+
+	vbox1 = gtk_vbox_new (FALSE, VSPACING);
+	gtk_widget_show (vbox1);
+	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
+
+	PACK_FRAME_WITH_CHECK_BUTTON(vbox1, socks_frame, socks_chkbtn,
+				     _("Use SOCKS proxy"));
+
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (socks_frame), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
+
+	hbox2 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox2);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, 0);
+
+	socks4_radiobtn = gtk_radio_button_new_with_label(NULL, "SOCKS4");
+	gtk_widget_show(socks4_radiobtn);
+	gtk_box_pack_start (GTK_BOX (hbox2), socks4_radiobtn, FALSE, FALSE, 0);
+	g_object_set_data(G_OBJECT(socks4_radiobtn), MENU_VAL_ID,
+			  GINT_TO_POINTER(SOCKS_SOCKS4));
+
+	CREATE_RADIO_BUTTON(hbox2, socks5_radiobtn, socks4_radiobtn, "SOCKS5",
+			    SOCKS_SOCKS5);
+
+	hbox2 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox2);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, 0);
+
+	label = gtk_label_new(_("Hostname:"));
+	gtk_widget_show(label);
+	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
+
+	socks_host_entry = gtk_entry_new();
+	gtk_widget_show(socks_host_entry);
+	gtk_widget_set_size_request(socks_host_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_box_pack_start(GTK_BOX(hbox2), socks_host_entry, TRUE, TRUE, 0);
+
+	label = gtk_label_new(_("Port:"));
+	gtk_widget_show(label);
+	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
+
+	socks_port_entry = gtk_entry_new();
+	gtk_widget_show(socks_port_entry);
+	gtk_widget_set_size_request(socks_port_entry, 64, -1);
+	gtk_box_pack_start(GTK_BOX(hbox2), socks_port_entry, FALSE, FALSE, 0);
+
+	vbox4 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox4);
+	gtk_box_pack_start(GTK_BOX(vbox2), vbox4, FALSE, FALSE, 0);
+
+	PACK_CHECK_BUTTON (vbox4, socks_auth_chkbtn, _("Use authentication"));
+
+	hbox2 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox2);
+	gtk_box_pack_start (GTK_BOX (vbox4), hbox2, FALSE, FALSE, 0);
+
+	label = gtk_label_new(_("Name:"));
+	gtk_widget_show(label);
+	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
+
+	socks_name_entry = gtk_entry_new();
+	gtk_widget_show(socks_name_entry);
+	gtk_widget_set_size_request(socks_name_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_box_pack_start(GTK_BOX(hbox2), socks_name_entry, TRUE, TRUE, 0);
+
+	label = gtk_label_new(_("Password:"));
+	gtk_widget_show(label);
+	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
+
+	socks_pass_entry = gtk_entry_new();
+	gtk_widget_show(socks_pass_entry);
+	gtk_widget_set_size_request(socks_pass_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_entry_set_visibility(GTK_ENTRY(socks_pass_entry), FALSE);
+	gtk_box_pack_start(GTK_BOX(hbox2), socks_pass_entry, TRUE, TRUE, 0);
+
+	PACK_CHECK_BUTTON(vbox2, socks_send_chkbtn,
+			  _("Use SOCKS proxy on sending"));
+
+	SET_TOGGLE_SENSITIVITY(socks_auth_chkbtn, hbox2);
+	SET_TOGGLE_SENSITIVITY(socks5_radiobtn, vbox4);
+	SET_TOGGLE_SENSITIVITY(socks_chkbtn, vbox2);
+
+	p_proxy.socks_chkbtn = socks_chkbtn;
+	p_proxy.socks4_radiobtn = socks4_radiobtn;
+	p_proxy.socks5_radiobtn = socks5_radiobtn;
+	p_proxy.socks_host_entry = socks_host_entry;
+	p_proxy.socks_port_entry = socks_port_entry;
+	p_proxy.socks_auth_chkbtn = socks_auth_chkbtn;
+	p_proxy.socks_name_entry = socks_name_entry;
+	p_proxy.socks_pass_entry = socks_pass_entry;
+	p_proxy.socks_send_chkbtn = socks_send_chkbtn;
+}
+
 static void prefs_account_advanced_create(void)
 {
 	GtkWidget *vbox1;
